from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap
from edge.webserver.sysconf.dgmod.mg.dgconf.mgd_unit import UnitConfBuilder
from edge.webserver.sysconf.dgmod.mg.dgconf.mgd_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.mg.uiconf import IUiConfProvider
from edge.webserver.sysconf.dgmod.mg.uiconf.fusion import FusionUiConfProvider
from edge.webserver.sysconf.dgmod.mg.uiconf.lpr import LprUiConfProvider
from edge.webserver.sysconf.dgmod.mg.uiconf.mv_camera import MvCameraUiConfProvider
from edge.webserver.sysconf.dgmod.mg.uiconf.radar import RadarUiConfProvider
from edge.webserver.sysconf.dgmod.mg.uiconf.redlight import RedlightUiConfProvider, RailroadUiConfProvider


# noinspection PyUnusedLocal
def mg_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                    all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                    uimod_to_newconf_map: UiModToConfMap,
                    forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type != ph.UI_TYPE_MV_CAMERA:
        return []

    try:
        cam_uimod, fusion_uimod, lpr_uimod, ra_uimod, rl_uimods, rc_uimods = (
            next(filter(lambda x: x.type == ph.UI_TYPE_MV_CAMERA, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods)),
            next(filter(lambda x: x.type == ph.UI_TYPE_RADAR, all_used_uimods), None),
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RED_LIGHT],
            [x for x in all_used_uimods if x.type == ph.UI_TYPE_RAILROAD_CROSSING])

        activate = any([
            uimod_to_newconf_map.get(cam_uimod),
            any([uimod_to_newconf_map.get(x) for x in rl_uimods]),
            any([uimod_to_newconf_map.get(x) for x in rc_uimods])
        ])
        return [MgCreator(cam_uimod, lpr_uimod, fusion_uimod, rl_uimods, rc_uimods, ra_uimod)]

    except StopIteration:
        pass

    return []


class MgCreator(BaseCreator):
    def __init__(self, cam_uimod: ph.UiMod,
                 lpr_uimod: ph.UiMod,
                 fusion_uimod: ph.UiMod,
                 rl_uimods: List[ph.UiMod],
                 rc_uimods: List[ph.UiMod],
                 ra_uimod: Optional[ph.UiMod]):

        super().__init__()

        self._dgmod_mg = dg.DgMod.create(dg.EDGE_TYPE_MEDIAGRABBER, cam_uimod.pid, cam_uimod.id)

        self._uimod_cam = cam_uimod
        self._uimod_fusion = fusion_uimod
        self._uimod_lpr = lpr_uimod
        self._uimod_ra = ra_uimod
        self._uimods_railroad = rc_uimods
        self._uimods_redlight = rl_uimods

        self._unit_builder = UnitConfBuilder(self._uimod_cam.id, ph.get_spool_dir(self._uimod_cam.pid))
        self._halt_builder = HaltConfBuilder()

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_mg

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        providers = list[IUiConfProvider]()

        uimod = self._uimod_cam
        providers.extend([
            MvCameraUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf()),
            FusionUiConfProvider(self._uimod_fusion.id),
            LprUiConfProvider(self._uimod_lpr.id)
        ])

        if self._uimod_ra:
            providers.extend([
                RadarUiConfProvider(self._uimod_ra.id)
            ])

        if len(self._uimods_redlight) == 1:
            uimod = self._uimods_redlight[0]
            providers.extend([
                RedlightUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        elif len(self._uimods_railroad) == 1:
            uimod = self._uimods_railroad[0]
            providers.extend([
                RailroadUiConfProvider(uimod.id, uimod_to_newconf_map.get(uimod) or uimod.load_conf())
            ])

        for provider in providers:
            provider.setup_builders(self._unit_builder, self._halt_builder)

    def complete(self) -> None:
        dgmod = self._dgmod_mg
        dgmod.install()

        self._unit_builder.build(self._dgmod_mg.conf_dir)
        dgmod.save_haltaction_conf(self._halt_builder.build())
        dgmod.save_setupstore_conf(active=True)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.sigint_dgmod(self.dgmod.type, self.dgmod.id)


class MgDeleter:
    def __init__(self, dgmod: dg.DgMod):

        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_MV_CAMERA:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_MEDIAGRABBER, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
